@@ -3,6 +3,7 @@ import logging
 import requests
 import aiohttp
 import socket
+import semver
 import time
 
 from palazzetti_sdk_asset_parser import AssetParser as psap
@@ -50,23 +51,22 @@ HUB_KEYS = [
     "IP",
 ]
 
-# to be completed!!
+
 class PalComm(object):
     async def async_callUDP(self, host, message):
         server = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
         server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
 
-        # Enable broadcasting mode
-        # server.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
-
         server.settimeout(DISCOVERY_TIMEOUT)
-        server.sendto(message, (host, UDP_PORT))
+        try:
+            server.sendto(message, (host, UDP_PORT))
+        except:
+            return
 
         while True:
             # Receive the client packet along with the address it is coming from
             try:
                 data, addr = server.recvfrom(BUFFER_SIZE)
-                # print(data.decode('utf-8'))
                 if data != "":
                     mydata = data.decode("utf-8")
                     mydata_json = json.loads(mydata)
@@ -78,14 +78,11 @@ class PalComm(object):
 
     async def async_getHTTP(self, host, message):
         queryStr = "http://" + host + "/cgi-bin/sendmsg.lua"
-        # params for GET
         params = (("cmd", message),)
         _response_json = None
 
-        # check if op is defined or stop here
         if message is None:
             return False
-        # print(message)
 
         _LOGGER.debug("Executing command: {message}")
 
@@ -193,7 +190,7 @@ class PalDiscovery(object):
                 return myips
 
     async def checkIP_UDP(self, testIP, response=False):
-        """verify the IP is a Connection Box using UDP"""
+        """verify if the IP is a Connection Box using UDP"""
 
         api_discovery = PalComm()
         use_ip = testIP
@@ -208,12 +205,12 @@ class PalDiscovery(object):
 
         return True
 
-    async def checkIP_HTTP(self, testIP, response=False):
+    async def checkIP_HTTP(self, testIP, response=False, message="GET STDT"):
         """verify the IP is a Connection Box using HTTP call with command GET LABL"""
         api_discovery = PalComm()
         use_ip = testIP
 
-        _response = await api_discovery.async_getHTTP(use_ip, "GET STDT")
+        _response = await api_discovery.async_getHTTP(use_ip, message)
 
         if not _response:
             return False
@@ -223,13 +220,13 @@ class PalDiscovery(object):
 
         return True
 
-    async def checkIP(self, testIP, response=False):
+    async def checkIP(self, testIP, response=False, message="GET STDT"):
         _response = await self.checkIP_UDP(testIP, response)
         # print(f"From checkIP_UDP {is_IP_OK}")
 
         if not _response:
-            # print("No ConnBox found via UDP, checking via HTTP...")
-            _response = await self.checkIP_HTTP(testIP, response)
+            # print("No Hub found via UDP, checking via HTTP...")
+            _response = await self.checkIP_HTTP(testIP, response, message)
             # print(f"From checkIP_HTTP {is_IP_OK}")
             if not _response:
                 # print("No ConnBox found")
@@ -264,8 +261,17 @@ class Hub(object):
         self._shape = None
         self.response_json = {"icon": "mdi:link-off", "IP": self.ip}
 
-    async def async_update(self, discovery=False):
-        _response = await self.paldiscovery.checkIP_UDP(self.ip, response=True)
+    async def async_update(self, discovery=False, deep=False, message="GET STDT"):
+        _response = None
+        _message = message
+        if deep:
+            # print("Deep discovery")
+            _response = await self.paldiscovery.checkIP(
+                self.ip, response=True, message=_message
+            )
+        else:
+            # print("UDP discovery")
+            _response = await self.paldiscovery.checkIP_UDP(self.ip, response=True)
         if not _response:
             self.online = False
             if self._product:
@@ -274,13 +280,6 @@ class Hub(object):
             await self.publish_updates()
             return
 
-        if discovery:
-            if not self._product:
-                self._product = Palazzetti(self.ip, self.unique_id + "_prd")
-            await self._product.async_get_stdt()
-            await self._product.async_get_alls()
-            await self._product.async_get_cntr()
-
         # merge the result with the exixting response_json
         if self.response_json != None:
             response_merged = self.response_json.copy()
@@ -293,6 +292,13 @@ class Hub(object):
         self.response_json.update({"icon": "mdi:link"})
         await self.publish_updates()
 
+        if discovery:
+            if not self._product:
+                self._product = Palazzetti(self.ip, self.unique_id + "_prd",self.hub_isbiocc)
+            await self._product.async_get_stdt()
+            await self._product.async_get_alls()
+            await self._product.async_get_cntr()
+
     @property
     def hub_online(self) -> bool:
         """Return unique ID of the gateway hub: ConnBox or BioCC"""
@@ -422,11 +428,12 @@ class Palazzetti(object):
         253: "NOPELLET ALARM",
     }
 
-    def __init__(self, host, product_id="uniqueID"):
+    def __init__(self, host, product_id="uniqueID",isbiocc=False):
         _LOGGER.debug("Init of class palazzetti")
 
         self.ip = host
         self.unique_id = product_id
+        self._isbiocc=isbiocc
 
         # internal variables
         self.palsocket = PalComm()
@@ -440,11 +447,21 @@ class Palazzetti(object):
         """Get STSTIC data via TCP call"""
         await self.__async_get_request("GET STDT")
 
+    async def async_get_lstd(self) -> None:
+        """Get STSTIC data via TCP call from Hub local file. Only for SYSTEM version >2.4.1"""
+        if semver.match(self.response_json["SYSTEM"].split(' ', 1)[0],'>2.4.1') and  not self._isbiocc:
+            await self.__async_get_request("GET LSTD")
+
     # TODO: Rename to get_alls
     async def async_get_alls(self) -> None:
         """Get LIVE data via TCP call"""
         await self.__async_get_request("GET ALLS")
 
+    async def async_get_lall(self) -> None:
+        """Get LIVE data via TCP call from Hub local file. Only for SYSTEM version >2.4.1"""
+        if semver.match(self.response_json["SYSTEM"].split(' ', 1)[0],'>2.4.1') and  not self._isbiocc:
+            await self.__async_get_request("GET LALL")
+
     async def async_UDP_get_alls(self) -> None:
         """Get LIVE data via UDP call"""
         await self.__async_UDP_get_request(b"plzbridge?GET ALLS")
@@ -883,7 +900,7 @@ class Palazzetti(object):
         if self.data_config_object.flag_has_switch_on_off != True:
             raise InvalidStateTransitionError
 
-        if self.data_config_object.value_product_is_on == False:
+        if self.data_config_object.value_product_is_on == True:
             raise InvalidStateTransitionError
 
         command = "CMD ON"
@@ -954,18 +971,15 @@ class Palazzetti(object):
     def get_data_stdt_json(self) -> dict:
         return self.response_json_stdt
 
-    # returns int with setpoint
     def get_setpoint(self) -> int:
-        """Get target temperature for climate"""
+        """Get setpoint temperature"""
         if self.response_json == None or self.response_json["SETP"] == None:
             return
 
         return int(self.response_json["SETP"])
 
-    # get generic KEY in the datas
-    # if key doesn't exist returns None
     def get_key(self, mykey="STATUS") -> str:
-        """Get target temperature for climate"""
+        """Get generic KEY in the data, returns None if key doesn't exist"""
         if (
             self.response_json == None
             or (mykey in self.response_json) == False
@@ -1005,4 +1019,4 @@ class Palazzetti(object):
     async def publish_updates(self) -> None:
         """Schedule call all registered callbacks."""
         for callback in self._callbacks:
-            callback()
\ No newline at end of file
+            callback()
